; ---------------------------------------------------------------------------------------------
; Scrolling test (SCROLLV.ASM) by Pere Serrat - v1.0 - 2022-11-24
; ---------------------------------------------------------------------------------------------
; Global definitions
; ---------------------------------------------------------------------------------------------
				org	$4000
				put	$4000
; ---------------------------------------------------------------------------------------------
; VDP hardware port addresses
; ---------------------------------------------------------------------------------------------
	IF MPI									; addresses for setups with MPI
VREG			equ	$ff56						; YM2413 register port for MPI
VDATA			equ	$ff57						; YM2413 data port for MPI
PDATA			equ	$ff58						; VRAM data - port #0
PCONF			equ	$ff59						; status, setup and VRAM address - port #1
PPAL			equ	$ff5a						; palette port (via Reg#16)- port #2
PINDA			equ	$ff5b						; indirect addressing (through Reg#17) - port #3
FMPORT		equ	$ff5c						; YM2149 register select port for MPI
FMDATA		equ	$ff5d						; YM2149 data port for MPI
SCREENCHG	equ	$ff5e						; screen switcher: 0=V9958 - 1=MC6847
	ELSE										; addresses for setups with OOJAMAFLIP
VREG			equ	$ff76						; YM2413 register port for MPI
VDATA			equ	$ff77						; YM2413 data port for MPI
PDATA			equ	$ff78						; VRAM data - port #0
PCONF			equ	$ff79						; status, setup and VRAM address - port #1
PPAL			equ	$ff7a						; palette port (via Reg#16)- port #2
PINDA			equ	$ff7b						; indirect addressing (through Reg#17) - port #3
FMPORT		equ	$ff7c						; YM2149 register select port for MPI
FMDATA		equ	$ff7d						; YM2149 data port for MPI
SCREENCHG	equ	$ff7e						; screen switcher: 0=V9958 - 1=MC6847
	ENDIF - MPI								; end sound addresses
; ---------------------------------------------------------------------------------------------
; 6809 Added constants, variables
; ---------------------------------------------------------------------------------------------
MPICFG		equ	$ff7f						; MPI config address
HOOKSINI		equ	$015e						; system Hooks beginning
HOOKSEND		equ	$01a9						; system Hooks end
INTDIS		equ	$50						; flags to disable interrupts
READKEY		equ	$A000						; indirect call to read keyboard (both machines)
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
oldSlot		fcb	$00						; active slot when program starts. Must not be initialized
flgSRead		fcb	$00						; flag to know if Int Status (S#0) has been read
; =============================================================================================
Start			orcc	#INTDIS					; disable interrupts
				lbsr	DetDos					; detach DOS
				lbsr  Set6309					; set native mode if a HD6309 is found
				lbsr	SetModeG4				; set graphic mode G4 (256x192 16 col bitmapped)
				lbsr	LoadPal					; Load colours palette
PgmBeg		lbsr	CfgHdw					; config Sound (6 bit DAC) and interrupts
												; enable VDP interrupts 
				lda	#$62 						;  0  BL IE0  M1  M2  0  SI MAG -> no screen, IE0
				sta	PCONF						; send value to port #1
				lda	#$81						; register #1
				sta	PCONF						; send register to port #1
; ---------------------------------------------------------------------------------------------
MLoop			jsr	[READKEY]				; read keyboard
				cmpa	#'Q'						; was 'Q' key pressed?
				beq	ExitGame					; yes, exit
				cmpa	#'S'						; was 'S' key pressed?
				bne	MLoop						; no, loopback
												; Scroll image 1 pixel upwards
				clra								; get value for R#23
				pshs	a							; save onto stack
; ---------------------------------------------------------------------------------------------
; Main loop
; ---------------------------------------------------------------------------------------------
DoScroll		sync								; wait for /FS
				sync								; wait for /FS
				lda	,s							; get counter
				inca								; increment it
				sta	,s							; update on stack
				lbsr	DisVdpInt				; disable interrupts
				sta	PCONF						; send number of 1st line to VDP
				lda	#$80+23					; R#23
				sta	PCONF						; to VDP
				lbsr	EnaVdpInt				; enable interrupts
				jsr	[READKEY]				; read keyboard
				cmpa	#'Q'						; was 'Q' key pressed?
				beq	ExitGame					; yes, exit
				bra	DoScroll					; loopback for next scroll
; ---------------------------------------------------------------------------------------------
; exit the program and restart the computer
; ---------------------------------------------------------------------------------------------
ExitGame		puls	a							; clean stack
				ldx	#PCONF					; point to port #1
				ldd	#$0081					; def. value for R#1, disabling screen
				sta	,x							; send value to port #1
				stb	,x							; send register to port #1
OldNMI		ldu   #$9d3d					; get the received NMI service routine address
				stu   $010a						; restore it
				ldmd	#$00						; enter emulation mode
				lda	oldSlot					; get received slot
				sta	MPICFG					; set it again+
				lda	#1							; value to
				sta	SCREENCHG				; set MC6847 screen
				clr	<$71						; set cold restart
				andcc	#$af						; enable interrupts
				jmp	[$fffe]					; restart computer
; ---------------------------------------------------------------------------------------------
; Set Graphic Mode G4
; Carefull. It doesn't enable SPRITES nor /FS Interrupts. This must be done later
; ---------------------------------------------------------------------------------------------
SetModeG4 	ldu	#settings				; point to settings to be sent to V9958
				ldb	#SETLEN					; number of bytes to be sent
				ldx	#PDATA					; point to Data port
				lbsr	DisVdpInt				; disable interrupts
SG01			lda	,u+						; get a byte, move pointer
				sta	1,x						; send to WP2+
				decb								; decrement counter
				bne	SG01						; not yet done? loopback
				lbsr	EnaVdpInt				; enable interrupts
				rts								; return
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
settings		fcb	$06,$80					;  0  DG IE2 IE1  M5  M4  M3   0 -> reset M5 and set M3,M4 (set G4 mode)
				fcb	$42,$81					;  0  BL IE0  M1  M2   0  SI MAG -> set BL,/MAG,/IE0 (screen enabled, interrupts disabled)
				fcb	$0a,$88					; MS  LP  TP  CB  VR   0 SPD  BW -> set /TP,VR,/SPD (VRAM 64kx1, sprites disabled)
				fcb	$00,$89					; LN   0  S1  S0  IL  E0 *NT  DC -> set 192 lines, NO interlace, no even/odd screens - NTSC
R7val			fcb	$00,$87					;TC3 TC2 TC1 TC0 BD3 BD2 BD1 BD0 -> set background/border colour (from palette), only low nibble works
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
				fcb	$1f,$82					;  0 A16 A15   1   1   1   1   1 -> Pattern Layout (A16-A15 for Page0) $00000		- %0000 0000 0000 0000
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
				fcb	$ef,$85					;A14 A13 A12 A11 A10   1   1   1 -> set A10-12-13-14-15 Sprite Attrib Table
				fcb	$01,$8b					;  0   0   0   0   0   0 A16 A15 ->	here $0f600 											- %1111 0110 0000 0000
													;											-> Sprite Colour Table is $200 above Attrib Table 
													;											-> here will be $0f400									
				fcb	$1f,$86					;  0   0 A16 A15 A14 A13 A12 A11 -> set A11-A12-13-14-15 Sprite PattGen  $0f800	- %1111 1000 0000 0000
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
R18val	
				fcb	$0e,$92					; V3  V2  V1  V0  H3  H2  H1  H0 -> center image horizontally - NTSC
SETLEN		equ	*-settings				; should be 20
; ---------------------------------------------------------------------------------------------
; Load colours palette. regX arrives pointing to port #0
; ---------------------------------------------------------------------------------------------
LoadPal		ldu	#defCols					; point to colours table
				ldd	#$0090					; palette register #0 to R#16
				sta	1,x						; send value to port #1
				stb	1,x						; send register to port #1
				ldb	#PALLEN					; number of bytes (16 colours x 2 bytes)
LPal01		lda	,u+						; get a byte
				sta	2,x						; send to port #2
				decb								; decrement counter
				bne	LPal01					; not done? loop
				rts								; return
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; MSX2+ palette without modifications
;					 	 RB  0G   RB  0G   RB  0G   RB  0G
defCols		fcb	$00,$00, $00,$00, $22,$05, $33,$06 	; colours 0  to 3
				fcb	$25,$02, $47,$03, $51,$02, $27,$06 	; colours 4  to 7
				fcb	$62,$03, $73,$04, $62,$06, $64,$06 	; colours 8  to 11
				fcb	$11,$04, $55,$03, $66,$06, $77,$07 	; colours 12 to 15
PALLEN		equ	*-defCols				; number of bytes
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;							  Red	  Blue Green	 New values	Expected value from Spectrum
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; 0 TRANSPARENT 			00		00		00  ->	0-0-0		--- not useful
; 1 BLACK					00		00		00  ->	0-0-0				00
; 2 BRIGHT GREEN			01		01		06  ->	1-1-7				67
; 3 LIGHT GREEN			03		03		07  ->	4-4-7				68
; 4 DEEP BLUE				01		07		01  ->   0-7-0				51
; 5 BRIGHT BLUE			02		07		03  ->	2-7-3				65
; 6 DEEP RED				05		01		01  ->	4-1-1				50
; 7 LIGHT BLUE				02		07		06  ->   0-7-6				53
; 8 BRIGHT RED				07		01		01  ->	7-0-0				66
; 9 LIGHT RED				07		03		03  ->	7-4-4				69
; A BRIGHT YELLOW			06		01		06  ->	7-0-7				54
; B PALE YELLOW			06		04		06  ->	7-4-7				70
; C DEEP GREEN				01		01		04  ->   1-1-4				52
; D DEEP PURPLE			06		05		02  ->   6-6-0				51
; E GREY						05		05		05  ->	4-4-4				55
; F WHITE					07		07		07  ->   7-7-7				71
; ------------------------------------------------------------------------------------------------------------------------------------------
; Enable/Disable V9958 /FS iterrupts but keeping screen enabled. Must save regAB
; ------------------------------------------------------------------------------------------------------------------------------------------
DisVdpInt	pshs	a							; save register A
				lda	#$20						; get a 'BRA' opcode 
				sta	>NMISwitch				; modify switch in NewNMI to SKIP S#0 process (disable INT)
				puls	a,pc						; restore register and return
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EnaVdpInt	pshs	a							; save register A
				tst	flgSRead					; is there an INT not yet aknowledged?
				beq	EnaVdpL01				; no, skip section
				ldd	#$008f					; ask for S#0 to R#15
				sta	PCONF						; send status register number
				stb	PCONF						; send register number to port #1
				lda	PCONF						; read port #1 to clean interrupt flag 
				clr	flgSRead					; reset flag to Status ALREADY read
EnaVdpL01	lda	#$21 						; get a 'BRN' opcode
				sta	>NMISwitch				; modify switch in NewNMI to ALLOW S#0 process (enable INT)
				puls	a,pc						; restore register and return
; ---------------------------------------------------------------------------------------------
; disables the D.O.S
; ---------------------------------------------------------------------------------------------
DetDos		lda	MPICFG					; get MPI current selected slot
				sta	oldSlot					; save to variable
				clr	MPICFG					; point to MSX2+ module
				ldx	#HOOKSINI				; point to hooks beginning
				lda	#$39						; opcode for RTS
DetD01		sta	,x+						; disable hoop
				cmpx	#HOOKSEND				; got to the end?
				blo	DetD01					; no, loopback
				rts								; return
; ---------------------------------------------------------------------------------------------
; config DAC sound and disable Sync detections
; ---------------------------------------------------------------------------------------------
CfgHdw		lda	$ff01						; get config PIA0-SideA
				anda	#%11110100				; reset LSB of MUX. Disable HS IRQ to CPU
				sta	$ff01						; update config PIA0-SideA
				lda	$ff03						; get config PIA0-SideB
				anda	#%11110100				; reset MSB of MUX -> %00 select DAC sound
				sta	$ff03						; update config PIA0-SideB
				clr	$ff20						; set zero sound volume
				lda	$ff23						; get config PIA1-SideB
				ora	#%00001000				; enable 6 bits DAC sound	
				sta	$ff23						; update config PIA1-SideB
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
												; substitute NMI service routine for a new one
				ldx   $010a						; get current NMI service routine
				stx   OldNMI+1					; save it to be restored when exiting game
				ldx  	#NewNMI					; point to new one
				stx   $010a						; set it
				rts								; return
; ---------------------------------------------------------------------------------------------
; new NMI Handler (for Vertical Blanking interrupts received from V9958)
; ---------------------------------------------------------------------------------------------
NewNMI		lda	#1							; set flag
				sta	flgSRead					; to Status NOT YET read
NMISwitch	brn	NewNMIEx					; self modifiable opcode to skip (or not) VDP S#0 process
				ldx	#PDATA					; point to VDP port #0
				ldd	#$008f					; ask for S#0 to R#15
				sta	1,x						; send status register number
				stb	1,x						; send register number to port #1
				lda	1,x						; read port #1 to clean flag 
				clr	flgSRead					; reset flag to Status ALREADY read
NewNMIEx		rti								; return from interrupt
; ---------------------------------------------------------------------------------------------
; If a HD6309 is found, enter native mode else exit game
; ---------------------------------------------------------------------------------------------
Set6309		ldw	#$a55a					; load the double register W = regE+regF
				tfr	w,x						; transfer to regX
				cmpx  #$a55a					; did it work?	(will fail on a 6809)
				bne   Not6309					; no, skip next
				ldmd  #$01 						; enter native mode
				rts								; return
Not6309		leas	2,s						; discard return address
				lbra	ExitGame					; end game. Not allowed for 6809 machines
; ---------------------------------------------------------------------------------------------
PgmEnd		equ	*
PgmLen		equ	PgmEnd-Start
; ---------------------------------------------------------------------------------------------
